/*
 * BMPReader.java
 *
 * Created on 20 de noviembre de 2003, 18:51
 */

package jmkirschberg.utils.image;
import jmkirschberg.utils.*;
import java.io.*;
import java.awt.image.*;
import java.util.*;
/**
 *
 * @author  Javier Muoz
 */
public class BMPReader {
    private static final int IDENTIFIER=0x424D;
    private static final int NO_COMPRESSION=0x0;
    private static int size;
    private static int width;
    private static int height;
    private static int planes;
    private static int bitsPerPixel;
    private static int compression;
    
    /** This function takes a DataInput containing a BMP file and produces a 
        Buffered Image
     *  @param data The DataInput
     *  @throws IOException whenever an IOException happens.
     *  @throws NotBMPFormatException when the file is not a correct .BMP Windows image
     */
    public static BufferedImage readBMP(DataInput data) throws
                  IOException,
                  NotBMPFormatException {
        int temp=data.readShort();
        int read=2;
        if(temp!=IDENTIFIER) throw new NotBMPFormatException();
        size=Util.readInt(data);
        read+=4;
        data.skipBytes(12);
        read+=12;
        width=Util.readInt(data);
        read+=4;
        height=Util.readInt(data);
        read+=4;
        /*width=data.readInt();
        height=data.readInt();*/
        BufferedImage image=new BufferedImage(width,height,BufferedImage.TYPE_3BYTE_BGR);
        WritableRaster raster=image.getRaster();
        planes=Util.readUnsignedShort(data);
        read+=2;
        bitsPerPixel=Util.readUnsignedShort(data);
        read+=2;
        compression=Util.readInt(data);
        read+=4;
        data.skipBytes(20);
        read+=20;
        if(bitsPerPixel==8) {
            //Palette: For each color the Red Green Blue scheme is given
            int[][] palette=new int[256][3];
            for(int i=0;i<256;i++) {
                palette[i][2]=data.readUnsignedByte();
                palette[i][1]=data.readUnsignedByte();
                palette[i][0]=data.readUnsignedByte();
                data.skipBytes(1);
                read+=4;
            }
            for(int i=0;i<width*height;i++) {
                /*System.out.println("i:"+i+","+(i%width)+","+(i/height));*/
                int tempColor=data.readUnsignedByte();
                read++;
                int[] tempArray = new int[3];
                tempArray[0]=palette[tempColor][0];
                tempArray[1]=palette[tempColor][1];
                tempArray[2]=palette[tempColor][2];
                raster.setPixel(i%width,i/width,tempArray);
            }
        } else if(bitsPerPixel==24) {
            int[] tempArray = new int[3];
            for(int i=0;i<width*height;i++) {
                tempArray[2]=data.readUnsignedByte();
                tempArray[1]=data.readUnsignedByte();
                tempArray[0]=data.readUnsignedByte();
                raster.setPixel(i%width,i/height,tempArray);
            }
        }
        return image;
    }
    
}

